using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Xml;
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using W3C.Soap;
using submgr = Microsoft.Dss.Services.SubscriptionManager;
using drive = Microsoft.Robotics.Services.Drive.Proxy;
using contactsensor = Microsoft.Robotics.Services.ContactSensor.Proxy;
using sonar = Microsoft.Robotics.Services.Sonar.Proxy;
using brightnesssensor = Microsoft.Robotics.Services.Simulation.Sensors.BrightnessSensor.Proxy;
using analogsensor = Microsoft.Robotics.Services.AnalogSensor.Proxy;
using Microsoft.Dss.Core;

namespace LegoSimulation
{
    [Contract(Contract.Identifier)]
    [DisplayName("LegoSimulation")]
    [Description("LegoSimulation service (no description provided)")]
    class LegoSimulationService : DsspServiceBase
    {
        /// <summary>
        /// Service state
        /// </summary>
        [ServiceState]
        LegoSimulationState _state = new LegoSimulationState();

        /// <summary>
        /// Main service port
        /// </summary>
        [ServicePort("/LegoSimulation", AllowMultipleInstances = true)]
        LegoSimulationOperations _mainPort = new LegoSimulationOperations();

        [SubscriptionManagerPartner]
        submgr.SubscriptionManagerPort _submgrPort = new submgr.SubscriptionManagerPort();

        /// <summary>
        /// DriveDifferentialTwoWheel partner
        /// </summary>
        [Partner("DriveDifferentialTwoWheel", Contract = drive.Contract.Identifier, CreationPolicy = PartnerCreationPolicy.UseExisting)]
        drive.DriveOperations _drivePort = new drive.DriveOperations();
        drive.DriveOperations _driveNotify = new drive.DriveOperations();

        /// <summary>
        /// ContactSensor partner
        /// </summary>
        [Partner("ContactSensor", Contract = contactsensor.Contract.Identifier, CreationPolicy = PartnerCreationPolicy.UseExisting)]
        contactsensor.ContactSensorArrayOperations _contactSensorPort = new contactsensor.ContactSensorArrayOperations();
        contactsensor.ContactSensorArrayOperations _contactSensorNotify = new contactsensor.ContactSensorArrayOperations();
        /// <summary>
        /// Sonar partner
        /// </summary>
        [Partner("Sonar", Contract = sonar.Contract.Identifier, CreationPolicy = PartnerCreationPolicy.UseExisting)]
        sonar.SonarOperations _sonarPort = new sonar.SonarOperations();
        sonar.SonarOperations _sonarNotify = new sonar.SonarOperations();

        /// <summary>
        /// SimulatedBrightnessCell partner
        /// </summary>
        [Partner("SimulatedBrightnessCell", Contract = brightnesssensor.Contract.Identifier, CreationPolicy = PartnerCreationPolicy.UseExisting)]
        analogsensor.AnalogSensorOperations _brightnesssensorPort = new analogsensor.AnalogSensorOperations();
        analogsensor.AnalogSensorOperations _brightnesssensorNotify = new analogsensor.AnalogSensorOperations();

        /// <summary>
        /// Service constructor
        /// </summary>
        public LegoSimulationService(DsspServiceCreationPort creationPort)
            : base(creationPort)
        {
        }

        /// <summary>
        /// Service start
        /// </summary>
        protected override void Start()
        {

            // 
            // Add service specific initialization here
            // 
            base.Start();
            _drivePort.Subscribe(_driveNotify);
            _contactSensorPort.Subscribe(_contactSensorNotify);
            _sonarPort.Subscribe(_sonarNotify);
            _brightnesssensorPort.Subscribe(_brightnesssensorNotify);
            Activate(
                Arbiter.Interleave(
                    new TeardownReceiverGroup(),
                    new ExclusiveReceiverGroup(),
                    new ConcurrentReceiverGroup(
                       Arbiter.Receive<drive.Update>(true, _driveNotify, DriveUpdateNotification),
                       //Arbiter.Receive<sonar.Replace>(true, _sonarNotify, SonarUpdateNotification),
                       Arbiter.Receive<analogsensor.Replace>(true, _brightnesssensorNotify, BrightUpdateNotification),
                       Arbiter.Receive<contactsensor.Update>(true, _contactSensorNotify, BumperHandler)
                    )
                )
            );
            _drivePort.SetDrivePower(0.15, 0.15);
        }
       
        private void BumperHandler(contactsensor.Update notification)
        {
            if (notification.Body.Pressed)
            {
                LogInfo(LogGroups.Console, "Ouch - the bumper was pressed.");
                _drivePort.SetDrivePower(0, 0);
                _drivePort.EnableDrive(false);
            }
        }

        void DriveUpdateNotification(drive.Update update)
        {
            Console.WriteLine("Update drive");
        }

        void SonarUpdateNotification(sonar.Replace update)
        {
            double r = update.Body.DistanceMeasurement;
            Console.WriteLine("Update sonar" + r.ToString());
        }

        void BrightUpdateNotification(analogsensor.Replace update)
        {
            double r = update.Body.RawMeasurement;
            Console.WriteLine("Update bright sensor" + r.ToString());
            if (r > 0.60) 
                _drivePort.SetDrivePower(-0.05, 0.05);
            else
                _drivePort.SetDrivePower(0.15, 0.15);
        }


        /// <summary>
        /// Handles Subscribe messages
        /// </summary>
        /// <param name="subscribe">the subscribe request</param>
        [ServiceHandler]
        public void SubscribeHandler(Subscribe subscribe)
        {
            SubscribeHelper(_submgrPort, subscribe.Body, subscribe.ResponsePort);
        }
    }
}


